package jp.co.intra_mart.cookbook.transfer_behind_logout;

import java.util.List;

import javax.servlet.jsp.JspException;

import jp.co.intra_mart.common.aid.jdk.java.lang.StringUtil;
import jp.co.intra_mart.foundation.context.Contexts;
import jp.co.intra_mart.foundation.context.model.AccountContext;
import jp.co.intra_mart.foundation.ui.tags.theme.UtilityItemProvider;

/**
 * ログイン・ログアウトを行うユーティリティプロバイダを提供します。<br>
 * アカウントコンテキストを利用して、リクエストを受け付けた際の認証状態に応じて以下のメニューを表示します。
 * <ul>
 * <li>{@link AccountContext#isAuthenticated()} がtrue(認証済み)である場合は、ログアウトを行うためのメニューを表示します。</li>
 * <li>{@link AccountContext#isAuthenticated()} がfalse(未認証)である場合は、ログインを行うためのメニューを表示します。</li>
 * </ul>
 * ログアウト後は {@link #TRANSITION_URL} に遷移します。
 * @author cookbook
 * @version 1.0.0
 */
public class LoginLogoutItemProvider implements UtilityItemProvider {

    /**
     * ログアウト後に表示するページのURL（URLエスケープ済み）
     */
    public static final String TRANSITION_URL = "http://www.intra-mart.jp";

    @Override
    public List<UtilityItemProvider> getChildren() throws JspException {
        // 子要素メニューは存在しません。
        return null;
    }

    /**
     * 認証済みである場合は、Logoutという文字列を返却します。<br>
     * 未認証である場合は、Loginというを返却します。
     * @return 認証状態に応じたメニューの表示文字列
     */
    @Override
    public String getLabel() throws JspException {
        if (isAuthenticated()) {
            return "Logout";
        }
        return "Login";
    }

    /**
     * 認証済みである場合は、ログアウトを行うためのURLを返却します。<br>
     * 未認証である場合は、ログインを行うためのURLを返却します。
     * @return 認証状態に応じたメニューのURL
     */
    @Override
    public String getUrl() throws JspException {
        if (isAuthenticated()) {
            // ログアウト時にリクエストパラメータを付与します。
            return "logout?im_url=" + TRANSITION_URL;
        }
        return "login";
    }

    @Override
    public boolean isVisible() throws JspException {
        // 常に有効（表示）します。
        return true;
    }

    @Override
    public String scriptSource() {
        // スクリプトは利用しません。
        return null;
    }

    @Override
    public String toListString() throws JspException {
        final StringBuilder buf = new StringBuilder();
        final String url = getUrl() == null ? StringUtil.EMPTY_STRING : "href=\"" + getUrl() + "\"";
        buf.append("<li><a ").append(url).append(">").append(getLabel()).append("</a>");
        if (getChildren() != null) {
            buf.append("<ul>");
            for (final UtilityItemProvider item : getChildren()) {
                buf.append(item.toListString());
            }
            buf.append("</ul>");
        }
        buf.append("</li>");
        return buf.toString();
    }

    /**
     * アカウントコンテキストを利用して認証状態を返します。
     * @return ログイン済みの場合 true
     */
    private boolean isAuthenticated() {
        return Contexts.get(AccountContext.class).isAuthenticated();
    }

}
